==> tecfy_core_package-0.1.95.tar.gz: lib/views/report/report_detail.page.dart <==
part of tecfy_core_client_package;

class ReportDetailPage extends StatefulWidget {
  final dynamic report;
  final List<EntityFilter> filter;
  ReportDetailPage(this.report, this.filter);
  @override
  _ReportDetailPageState createState() => _ReportDetailPageState();
}

class _ReportDetailPageState extends AppState<ReportDetailPage> {
  List<dynamic> data;
  dynamic totals;
  int sortColumnIndex;
  bool sortAcd = true;
  ByteData font;
  List<dynamic> props;
  ScrollController _scrollControllerH;
  ScrollController _scrollControllerV;
  @override
  void initState() {
    _scrollControllerV = ScrollController();
    _scrollControllerH = ScrollController();
    rootBundle.load("assets/fonts/STC-Regular.ttf").then((d) => font = d);
    props = widget.report['properties'];
    if (LocalConfigService.isArabic) props = props.reversed.toList();
    getData();
    super.initState();
  }

  @override
  void dispose() {
    _scrollControllerH.dispose();
    _scrollControllerV.dispose();
    super.dispose();
  }

  void onSort(String dbName, int type) {
    data.sort((a, b) => sortAcd
        ? (type == 3
            ? (a[dbName] - b[dbName]).toInt()
            : a[dbName].toString().compareTo(b[dbName].toString()))
        : (type == 3
            ? (b[dbName] - a[dbName]).toInt()
            : b[dbName].toString().compareTo(a[dbName].toString())));
    setState(() {});
  }

  getData() async {
    try {
      //List<dynamic> data;
      // if (ConnectionService.isOfflineMode) {
      //   var result = await EntityService().getData(
      //       EntityService.getEntity(widget.report['entityName']),
      //       filter: widget.filter,
      //       allRecords: true);
      //   data = result.data;
      // } else {
      if (TecfyCoreApp.reportService != null &&
          TecfySyncService.isOfflineMode &&
          widget.report['isOnlineOnly'] == false) {
        data = await TecfyCoreApp.reportService
            .getReportData(widget.report['dbName'], widget.filter);
      } else {
        var result = await TecfyBasicApp.apiService.httpGetDynamic(
            'Report/GetData/${widget.report['dbName']}' +
                ((widget.filter?.length ?? 0) > 0
                    ? '?filter=${jsonEncode(widget.filter.map<Map>((f) => f.toJson()).toList(), toEncodable: EntityService.customEncode)}'
                    : ''));
        data = result.data;
      }
      // }
      if (data != null) {
        // if (widget.report['totals'] != null &&
        //     widget.report['totals'].length > 0) {
        //   totals = {};
        //   for (var t in widget.report['totals']) {
        //     totals[t] = 0;
        //     for (var d in data) {
        //       totals[t] += d[t] ?? 0;
        //     }
        //   }
        // }
        totals = data.removeLast();
        setState(() {});
      } else {
        await AppNavigator.showMessage(context, "Error!");
        Navigator.pop(context);
      }
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXX report load error');
      print(err);
      print(trace);
      await AppNavigator.showMessage(context, "Error getting data!");
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      widget.report['name${LocalConfigService.languagePrefex}'],
      appBarActions: [
        AppButtonIconText(
          label: 'Print',
          icon: Icon(Icons.print, color: Colors.white),
          onPressed: onPrint,
        ),
        SizedBox(width: 5),
        AppButtonIconText(
          label: 'PDF',
          icon: Icon(Icons.picture_as_pdf, color: Colors.white),
          onPressed: onSharePDF,
        )
      ],
      child: pageContent(),
    );
  }

  Future<dynamic> onPrint() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    var branchLogo =
        await ImageService.getImageBytes(resultData.data[1]["logoId"]);
    var doc = getPrintablePdf(
        resultData.data[1]["printTitleAr"],
        resultData.data[1]["printSubTitleAr"],
        resultData.data[1]["printFooterAr"],
        resultData.data[1]["printSubFooterAr"],
        branchLogo);
    var result = await Printing.layoutPdf(
      onLayout: (format) async => doc.save(),
      //name:widget.report['name${LocalConfigService.languagePrefex}'],
    );
    print(result);
  }

  Future<dynamic> onSharePDF() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    var branchLogo =
        await ImageService.getImageBytes(resultData.data[1]["logoId"]);
    var doc = getPrintablePdf(
      resultData.data[1]["printTitleAr"],
      resultData.data[1]["printSubTitleAr"],
      resultData.data[1]["printFooterAr"],
      resultData.data[1]["printSubFooterAr"],
      branchLogo,
    );
    var result = await Printing.sharePdf(
        bytes: await doc.save(), filename: 'nameEn.pdf');
    // var result = await Printing.layoutPdf(
    //   onLayout: (format) async => doc.save(),
    //   //name:widget.report['name${LocalConfigService.languagePrefex}'],
    // );
    print(result);
  }

  pw.Document getPrintablePdf(
      var titleAr, var subTitleAr, var footerAr, var subFooterAr, var logoId) {
    final doc = pw.Document();
    DateTime printingDate = DateTime.now();
    String formattedDate =
        intl.DateFormat('yyyy-MM-dd   kk:mm').format(printingDate);
    doc.addPage(
      pw.MultiPage(
        pageFormat: PdfPageFormat.a4,
        textDirection: pw.TextDirection.rtl,
        footer: (context) =>
            pw.Row(mainAxisAlignment: pw.MainAxisAlignment.center, children: [
          pw.Row(children: [
            pw.Text(" ${formattedDate.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.ltr),
            pw.Text('تاريخ الطباعة : ',
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
          ]),
          pw.SizedBox(width: 40),
          pw.Row(children: [
            pw.Text(" ${LocalConfigService.user.fullName.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
            pw.Text('أسم المستخدم : ',
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
          ]),
        ]),
        build: (c) => [
          pw.Padding(
              padding: pw.EdgeInsets.only(top: 0),
              child: pw.Column(children: [
                pw.Row(children: [
                  pw.Expanded(
                      child: pw.Row(children: [
                    pw.Container(
                      width: 120,
                      height: 120,
                      decoration: pw.BoxDecoration(
                          image: pw.DecorationImage(
                              image: pw.MemoryImage(logoId),
                              fit: pw.BoxFit.contain)),
                    ),
                    pw.Container(width: 40),
                  ])),
                  pw.SizedBox(width: 20),
                  pw.Expanded(
                      child: pw.Column(
                    crossAxisAlignment: pw.CrossAxisAlignment.end,
                    children: [
                      pw.SizedBox(height: 5),
                      pw.Text(
                        titleAr.toString(),
                        style: pw.TextStyle(
                            font: pw.Font.ttf(font),
                            fontWeight: pw.FontWeight.bold,
                            fontSize: 16),
                        // textDirection: pw.TextDirection.rtl
                      ),
                      pw.Text(subTitleAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(footerAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(subFooterAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                    ],
                  )),
                ]),
              ])),
          pw.Center(
              child: pw.Text(
                  widget.report['name${LocalConfigService.languagePrefex}'],
                  style: pw.TextStyle(font: pw.Font.ttf(font), fontSize: 15))),
          pw.SizedBox(height: 20),
          pw.Table(
              //columnWidths: sortColumnIndex,
              border: pw.TableBorder(
                  horizontalInside: pw.BorderSide(),
                  top: pw.BorderSide(),
                  bottom: pw.BorderSide(),
                  right: pw.BorderSide(),
                  left: pw.BorderSide()),
              children: data
                  .map<pw.TableRow>((d) => pw.TableRow(
                        children: props
                            .map<pw.Widget>((p) => getCellReport(p, d))
                            .toList(),
                      ))
                  .toList()
                ..insert(
                  0,
                  pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey300),
                    children: props
                        .map<pw.Widget>(
                          (p) => pw.Padding(
                            padding: pw.EdgeInsets.only(left: 5, right: 5),
                            child: pw.Align(
                              alignment: LocalConfigService.isArabic
                                  ? pw.Alignment.centerRight
                                  : pw.Alignment.centerLeft,
                              child: pw.Text(
                                p['name${LocalConfigService.languagePrefex}'] ??
                                    'N/A',
                                style: pw.TextStyle(
                                  font: pw.Font.ttf(font),
                                ),
                              ),
                            ),
                          ),
                        )
                        .toList(),
                  ),
                )
                ..add(pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey200),
                    children: props
                        .map<pw.Widget>(
                            (p) => getCellReport(p, totals, totals: true))
                        .toList()))),
        ],
      ),
    );

    return doc;
  }

  pw.Widget getCellReport(prop, data, {totals: false}) {
    var val = data[prop['dbName']];

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return pw.Container();
    } else {
      if (val == null) return pw.Container();
    }
    var textStyle = totals
        ? pw.TextStyle(
            font: pw.Font.ttf(font),
            fontSize: 13,
            fontWeight: pw.FontWeight.bold)
        : pw.TextStyle(font: pw.Font.ttf(font));
    pw.Widget cell;
    switch (ValueTypes.values[prop['type'] as int]) {
      case ValueTypes.Text:
        cell = pw.Text(val, style: textStyle);
        break;
      case ValueTypes.Number:
        cell = pw.Text((val as num).toDouble().toStringAsFixed(2),
            style: textStyle);
        break;
      case ValueTypes.DateTime:
        DateTime date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val).toLocal();
        if (val is String) date = BaseModel.parseUTC(val);
        if (val is DateTime) date = val;

        cell = pw.Text(
            intl.DateFormat(prop['format'] ?? "yyyy-MM-dd HH:mm")
                .format(date.toLocal()),
            style: textStyle);

        // cell = pw.Text(val.toString().replaceFirst('T', ' ').substring(0, 16),
        //     style: textStyle);
        break;
      default:
        cell = pw.Text('-');
        break;
    }
    return pw.Padding(
        padding: pw.EdgeInsets.only(left: 5, right: 5),
        child: pw.Align(
            alignment: LocalConfigService.isArabic
                ? pw.Alignment.centerRight
                : pw.Alignment.centerLeft,
            child: cell));
  }

  Widget pageContent() {
    return data == null
        ? Center(
            child: CircularProgressIndicator(
                valueColor:
                    AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
        // : data != null
        //     ? PdfPreview(build: (c) => getPrintablePdf().save())
        : Scrollbar(
            isAlwaysShown: true,
            showTrackOnHover: true,
            controller: _scrollControllerH,
            child: SingleChildScrollView(
              controller: _scrollControllerH,
              scrollDirection: Axis.horizontal,
              child: Scrollbar(
                isAlwaysShown: true,
                showTrackOnHover: true,
                controller: _scrollControllerV,
                child: SingleChildScrollView(
                  controller: _scrollControllerV,
                  scrollDirection: Axis.vertical,
                  child: DataTable(
                      showCheckboxColumn: true,
                      sortColumnIndex: sortColumnIndex,
                      sortAscending: sortAcd,
                      columns: widget.report['properties']
                          .map<DataColumn>((p) => DataColumn(
                              label: Text(
                                  p['name${LocalConfigService.languagePrefex}'] ??
                                      'N/A'),
                              numeric: p['type'] == 3,
                              onSort: (s, d) {
                                sortColumnIndex = s;
                                sortAcd = d;
                                onSort(p['dbName'], p['type']);
                              },
                              tooltip: p['nameEn'] ?? p['nameAr']))
                          .toList(),
                      rows: data
                          .map<DataRow>((d) => DataRow(
                                cells: widget.report['properties']
                                    .map<DataCell>((p) => getCell(p, d))
                                    .toList(),
                              ))
                          .toList()
                        ..add(DataRow(
                          cells: widget.report['properties']
                              .map<DataCell>(
                                  (p) => getCell(p, totals, totals: true))
                              .toList(),
                        ))
                      // ..insert(0, TableRow(
                      //   children: widget.report['properties'].map<TableCell>((p)=>TableCell(child: Text(p['name${LocalConfigService.languagePrefex}'] ?? 'N/A', style: ThemeService.titleStyle))).toList(),
                      // )
                      //)
                      ),
                ),
              ),
            ),
          );
  }

  DataCell getCell(prop, data, {totals: false}) {
    var val = data[prop['dbName']];

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return DataCell(Container());
    } else {
      if (val == null) return DataCell(Text('.'));
    }
    switch (ValueTypes.values[prop['type'] as int]) {
      case ValueTypes.Text:
        return DataCell(Text(val,
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.Number:
        return DataCell(Text((val as num).toDouble().toStringAsFixed(2),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.DateTime:
        DateTime date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val).toLocal();
        if (val is String) date = BaseModel.parseUTC(val);
        if (val is DateTime) date = val;

        return DataCell(Text(
            intl.DateFormat(prop['format'] ?? "yyyy-MM-dd HH:mm")
                .format(date.toLocal()),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10)));
      default:
        return DataCell(Text('-'));
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}

==> tecfy_core_package-0.1.100.tar.gz: lib/views/report/report_detail.page.dart <==
part of tecfy_core_client_package;

class ReportDetailPage extends StatefulWidget {
  final dynamic report;
  final List<EntityFilter> filter;
  ReportDetailPage(this.report, this.filter);
  @override
  _ReportDetailPageState createState() => _ReportDetailPageState();
}

class _ReportDetailPageState extends AppState<ReportDetailPage> {
  List<dynamic> data;
  dynamic totals;
  int sortColumnIndex;
  bool sortAcd = true;
  ByteData font;
  List<dynamic> props;
  ScrollController _scrollControllerH;
  ScrollController _scrollControllerV;
  @override
  void initState() {
    _scrollControllerV = ScrollController();
    _scrollControllerH = ScrollController();
    rootBundle.load("assets/fonts/STC-Regular.ttf").then((d) => font = d);
    props = widget.report['properties'];
    if (LocalConfigService.isArabic) props = props.reversed.toList();
    getData();
    super.initState();
  }

  @override
  void dispose() {
    _scrollControllerH.dispose();
    _scrollControllerV.dispose();
    super.dispose();
  }

  void onSort(String dbName, int type) {
    data.sort((a, b) => sortAcd
        ? (type == 3
            ? (a[dbName] - b[dbName]).toInt()
            : a[dbName].toString().compareTo(b[dbName].toString()))
        : (type == 3
            ? (b[dbName] - a[dbName]).toInt()
            : b[dbName].toString().compareTo(a[dbName].toString())));
    setState(() {});
  }

  getData() async {
    try {
      //List<dynamic> data;
      // if (ConnectionService.isOfflineMode) {
      //   var result = await EntityService().getData(
      //       EntityService.getEntity(widget.report['entityName']),
      //       filter: widget.filter,
      //       allRecords: true);
      //   data = result.data;
      // } else {
      if (TecfyCoreApp.reportService != null &&
          TecfySyncService.isOfflineMode &&
          widget.report['isOnlineOnly'] == false) {
        data = await TecfyCoreApp.reportService
            .getReportData(widget.report['dbName'], widget.filter);
      } else {
        var result = await TecfyBasicApp.apiService.httpGetDynamic(
            'Report/GetData/${widget.report['dbName']}' +
                ((widget.filter?.length ?? 0) > 0
                    ? '?filter=${jsonEncode(widget.filter.map<Map>((f) => f.toJson()).toList(), toEncodable: EntityService.customEncode)}'
                    : ''));
        data = result.data;
      }
      // }
      if (data != null) {
        // if (widget.report['totals'] != null &&
        //     widget.report['totals'].length > 0) {
        //   totals = {};
        //   for (var t in widget.report['totals']) {
        //     totals[t] = 0;
        //     for (var d in data) {
        //       totals[t] += d[t] ?? 0;
        //     }
        //   }
        // }
        totals = data.removeLast();
        setState(() {});
      } else {
        await AppNavigator.showMessage(context, "Error or No Data!");
        Navigator.pop(context);
      }
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXX report load error');
      print(err);
      print(trace);
      await AppNavigator.showMessage(context, "Error getting data!");
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      widget.report['name${LocalConfigService.languagePrefex}'],
      appBarActions: [
        AppButtonIconText(
          label: 'Print',
          icon: Icon(Icons.print, color: Colors.white),
          onPressed: onPrint,
        ),
        SizedBox(width: 5),
        AppButtonIconText(
          label: 'PDF',
          icon: Icon(Icons.picture_as_pdf, color: Colors.white),
          onPressed: onSharePDF,
        )
      ],
      child: pageContent(),
    );
  }

  Future<dynamic> onPrint() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    var branchLogo =
        await ImageService.getImageBytes(resultData.data[1]["logoId"]);
    var doc = getPrintablePdf(
        resultData.data[1]["printTitleAr"],
        resultData.data[1]["printSubTitleAr"],
        resultData.data[1]["printFooterAr"],
        resultData.data[1]["printSubFooterAr"],
        branchLogo);
    var result = await Printing.layoutPdf(
      onLayout: (format) async => doc.save(),
      //name:widget.report['name${LocalConfigService.languagePrefex}'],
    );
    print(result);
  }

  Future<dynamic> onSharePDF() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    var branchLogo =
        await ImageService.getImageBytes(resultData.data[1]["logoId"]);
    var doc = getPrintablePdf(
      resultData.data[1]["printTitleAr"],
      resultData.data[1]["printSubTitleAr"],
      resultData.data[1]["printFooterAr"],
      resultData.data[1]["printSubFooterAr"],
      branchLogo,
    );
    var result = await Printing.sharePdf(
        bytes: await doc.save(), filename: 'nameEn.pdf');
    // var result = await Printing.layoutPdf(
    //   onLayout: (format) async => doc.save(),
    //   //name:widget.report['name${LocalConfigService.languagePrefex}'],
    // );
    print(result);
  }

  pw.Document getPrintablePdf(
      var titleAr, var subTitleAr, var footerAr, var subFooterAr, var logoId) {
    final doc = pw.Document();
    DateTime printingDate = DateTime.now();
    String formattedDate =
        intl.DateFormat('yyyy-MM-dd   kk:mm').format(printingDate);
    doc.addPage(
      pw.MultiPage(
        pageFormat: PdfPageFormat.a4,
        textDirection: pw.TextDirection.rtl,
        footer: (context) =>
            pw.Row(mainAxisAlignment: pw.MainAxisAlignment.center, children: [
          pw.Row(children: [
            pw.Text(" ${formattedDate.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.ltr),
            pw.Text('تاريخ الطباعة : ',
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
          ]),
          pw.SizedBox(width: 40),
          pw.Row(children: [
            pw.Text(" ${LocalConfigService.user.fullName.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
            pw.Text('أسم المستخدم : ',
                style: pw.TextStyle(font: pw.Font.ttf(font)),
                textDirection: pw.TextDirection.rtl),
          ]),
        ]),
        build: (c) => [
          pw.Padding(
              padding: pw.EdgeInsets.only(top: 0),
              child: pw.Column(children: [
                pw.Row(children: [
                  pw.Expanded(
                      child: pw.Row(children: [
                    pw.Container(
                      width: 120,
                      height: 120,
                      decoration: pw.BoxDecoration(
                          image: pw.DecorationImage(
                              image: pw.MemoryImage(logoId),
                              fit: pw.BoxFit.contain)),
                    ),
                    pw.Container(width: 40),
                  ])),
                  pw.SizedBox(width: 20),
                  pw.Expanded(
                      child: pw.Column(
                    crossAxisAlignment: pw.CrossAxisAlignment.end,
                    children: [
                      pw.SizedBox(height: 5),
                      pw.Text(
                        titleAr.toString(),
                        style: pw.TextStyle(
                            font: pw.Font.ttf(font),
                            fontWeight: pw.FontWeight.bold,
                            fontSize: 16),
                        // textDirection: pw.TextDirection.rtl
                      ),
                      pw.Text(subTitleAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(footerAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(subFooterAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(font)),
                          textDirection: pw.TextDirection.rtl),
                    ],
                  )),
                ]),
              ])),
          pw.Center(
              child: pw.Text(
                  widget.report['name${LocalConfigService.languagePrefex}'],
                  style: pw.TextStyle(font: pw.Font.ttf(font), fontSize: 15))),
          pw.SizedBox(height: 20),
          pw.Table(
              //columnWidths: sortColumnIndex,
              border: pw.TableBorder(
                  horizontalInside: pw.BorderSide(),
                  top: pw.BorderSide(),
                  bottom: pw.BorderSide(),
                  right: pw.BorderSide(),
                  left: pw.BorderSide()),
              children: data
                  .map<pw.TableRow>((d) => pw.TableRow(
                        children: props
                            .map<pw.Widget>((p) => getCellReport(p, d))
                            .toList(),
                      ))
                  .toList()
                ..insert(
                  0,
                  pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey300),
                    children: props
                        .map<pw.Widget>(
                          (p) => pw.Padding(
                            padding: pw.EdgeInsets.only(left: 5, right: 5),
                            child: pw.Align(
                              alignment: LocalConfigService.isArabic
                                  ? pw.Alignment.centerRight
                                  : pw.Alignment.centerLeft,
                              child: pw.Text(
                                p['name${LocalConfigService.languagePrefex}'] ??
                                    'N/A',
                                style: pw.TextStyle(
                                  font: pw.Font.ttf(font),
                                ),
                              ),
                            ),
                          ),
                        )
                        .toList(),
                  ),
                )
                ..add(pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey200),
                    children: props
                        .map<pw.Widget>(
                            (p) => getCellReport(p, totals, totals: true))
                        .toList()))),
        ],
      ),
    );

    return doc;
  }

  pw.Widget getCellReport(prop, data, {totals: false}) {
    var val = data[prop['dbName']];

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return pw.Container();
    } else {
      if (val == null) return pw.Container();
    }
    var textStyle = totals
        ? pw.TextStyle(
            font: pw.Font.ttf(font),
            fontSize: 13,
            fontWeight: pw.FontWeight.bold)
        : pw.TextStyle(font: pw.Font.ttf(font));
    pw.Widget cell;
    switch (ValueTypes.values[prop['type'] as int]) {
      case ValueTypes.Text:
        cell = pw.Text(val, style: textStyle);
        break;
      case ValueTypes.Number:
        cell = pw.Text((val as num).toDouble().toStringAsFixed(2),
            style: textStyle);
        break;
      case ValueTypes.DateTime:
        DateTime date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val).toLocal();
        if (val is String) date = BaseModel.parseUTC(val);
        if (val is DateTime) date = val;

        cell = pw.Text(
            intl.DateFormat(prop['format'] ?? "yyyy-MM-dd HH:mm")
                .format(date.toLocal()),
            style: textStyle);

        // cell = pw.Text(val.toString().replaceFirst('T', ' ').substring(0, 16),
        //     style: textStyle);
        break;
      default:
        cell = pw.Text('-');
        break;
    }
    return pw.Padding(
        padding: pw.EdgeInsets.only(left: 5, right: 5),
        child: pw.Align(
            alignment: LocalConfigService.isArabic
                ? pw.Alignment.centerRight
                : pw.Alignment.centerLeft,
            child: cell));
  }

  Widget pageContent() {
    return data == null
        ? Center(
            child: CircularProgressIndicator(
                valueColor:
                    AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
        // : data != null
        //     ? PdfPreview(build: (c) => getPrintablePdf().save())
        : Scrollbar(
            isAlwaysShown: true,
            showTrackOnHover: true,
            controller: _scrollControllerH,
            child: SingleChildScrollView(
              controller: _scrollControllerH,
              scrollDirection: Axis.horizontal,
              child: Scrollbar(
                isAlwaysShown: true,
                showTrackOnHover: true,
                controller: _scrollControllerV,
                child: SingleChildScrollView(
                  controller: _scrollControllerV,
                  scrollDirection: Axis.vertical,
                  child: DataTable(
                      showCheckboxColumn: true,
                      sortColumnIndex: sortColumnIndex,
                      sortAscending: sortAcd,
                      columns: widget.report['properties']
                          .map<DataColumn>((p) => DataColumn(
                              label: Text(
                                  p['name${LocalConfigService.languagePrefex}'] ??
                                      'N/A'),
                              numeric: p['type'] == 3,
                              onSort: (s, d) {
                                sortColumnIndex = s;
                                sortAcd = d;
                                onSort(p['dbName'], p['type']);
                              },
                              tooltip: p['nameEn'] ?? p['nameAr']))
                          .toList(),
                      rows: data
                          .map<DataRow>((d) => DataRow(
                                cells: widget.report['properties']
                                    .map<DataCell>((p) => getCell(p, d))
                                    .toList(),
                              ))
                          .toList()
                        ..add(DataRow(
                          cells: widget.report['properties']
                              .map<DataCell>(
                                  (p) => getCell(p, totals, totals: true))
                              .toList(),
                        ))
                      // ..insert(0, TableRow(
                      //   children: widget.report['properties'].map<TableCell>((p)=>TableCell(child: Text(p['name${LocalConfigService.languagePrefex}'] ?? 'N/A', style: ThemeService.titleStyle))).toList(),
                      // )
                      //)
                      ),
                ),
              ),
            ),
          );
  }

  DataCell getCell(prop, data, {totals: false}) {
    var val = data[prop['dbName']];

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return DataCell(Container());
    } else {
      if (val == null) return DataCell(Text('.'));
    }
    switch (ValueTypes.values[prop['type'] as int]) {
      case ValueTypes.Text:
        return DataCell(Text(val,
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.Number:
        return DataCell(Text((val as num).toDouble().toStringAsFixed(2),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.DateTime:
        DateTime date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val).toLocal();
        if (val is String) date = BaseModel.parseUTC(val);
        if (val is DateTime) date = val;

        return DataCell(Text(
            intl.DateFormat(prop['format'] ?? "yyyy-MM-dd HH:mm")
                .format(date.toLocal()),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10)));
      default:
        return DataCell(Text('-'));
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}
